# encoding: utf-8
'''
Taken from http://stackoverflow.com/questions/384076/how-can-i-make-the-python-logging-output-to-be-colored
'''
import logging

SUPPORTED_PLATFORMS = ('linux', 'darwin',)

#The background is set with 40 plus the number of the color, and the foreground with 30

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = xrange(30, 38)

# Sequences 
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"

COLORS = {
    'CRITICAL': MAGENTA, # level 50
    'ERROR': RED,        # level 40
    'WARNING': CYAN,  # level 30
    'INFO': GREEN,        # level 20
    'DEBUG': BLUE,       # level 10
    'SQL' : YELLOW,      # level 5 (default, can be modified)
}

# now we patch Python code to add color support to logging.StreamHandler
def add_coloring_to_emit_windows(fn):
        # add methods we need to the class
    def _out_handle(self):
        import ctypes
        return ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
    out_handle = property(_out_handle)

    def _set_color(self, code):
        import ctypes
        # Constants from the Windows API
        self.STD_OUTPUT_HANDLE = -11
        hdl = ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
        ctypes.windll.kernel32.SetConsoleTextAttribute(hdl, code)

    setattr(logging.StreamHandler, '_set_color', _set_color)

    def new(*args):
        # winbase.h
        STD_INPUT_HANDLE = -10
        STD_OUTPUT_HANDLE = -11
        STD_ERROR_HANDLE = -12

        # wincon.h
        FOREGROUND_BLACK     = 0x0000
        FOREGROUND_BLUE      = 0x0001
        FOREGROUND_GREEN     = 0x0002
        FOREGROUND_CYAN      = 0x0003
        FOREGROUND_RED       = 0x0004
        FOREGROUND_MAGENTA   = 0x0005
        FOREGROUND_YELLOW    = 0x0006
        FOREGROUND_GREY      = 0x0007
        FOREGROUND_INTENSITY = 0x0008 # foreground color is intensified.
        FOREGROUND_WHITE     = FOREGROUND_BLUE|FOREGROUND_GREEN |FOREGROUND_RED

        BACKGROUND_BLACK     = 0x0000
        BACKGROUND_BLUE      = 0x0010
        BACKGROUND_GREEN     = 0x0020
        BACKGROUND_CYAN      = 0x0030
        BACKGROUND_RED       = 0x0040
        BACKGROUND_MAGENTA   = 0x0050
        BACKGROUND_YELLOW    = 0x0060
        BACKGROUND_GREY      = 0x0070
        BACKGROUND_INTENSITY = 0x0080 # background color is intensified.     

        levelno = args[1].levelno
        if levelno >= 50:
            color = BACKGROUND_YELLOW | FOREGROUND_RED | FOREGROUND_INTENSITY | BACKGROUND_INTENSITY 
        elif levelno >= 40:
            color = FOREGROUND_RED | FOREGROUND_INTENSITY
        elif levelno >= 30:
            color = FOREGROUND_YELLOW | FOREGROUND_INTENSITY
        elif levelno >= 20:
            color = FOREGROUND_GREEN
        elif levelno >= 10:
            color = FOREGROUND_MAGENTA
        elif levelno >= 5:
            color = FOREGROUND_YELLOW
        else:
            color =  FOREGROUND_WHITE
        args[0]._set_color(color)

        ret = fn(*args)
        args[0]._set_color( FOREGROUND_WHITE )
        #print "after"
        return ret
    return new

def add_coloring_to_emit_ansi(fn):
    # add methods we need to the class
    def new(*args):
        levelno = args[1].levelno
        if levelno >= 50:
            color = COLOR_SEQ % COLORS['CRITICAL']
        elif levelno >= 40:
            color = COLOR_SEQ % COLORS['ERROR']
        elif levelno >= 30:
            color = COLOR_SEQ % COLORS['WARNING']
        elif levelno >= 20:
            color = COLOR_SEQ % COLORS['DEBUG']
        elif levelno >= 10:
            color = COLOR_SEQ % COLORS['INFO']
        elif levelno >= 5:
            color = COLOR_SEQ % COLORS['SQL'] 
        else:
            color = '\x1b[0m' # normal
        args[1].msg = color + args[1].msg +  '\x1b[0m'  # normal
        return fn(*args)
    return new

'''
import platform

if platform.system()=='Windows':
    logging.StreamHandler.emit = add_coloring_to_emit_windows(logging.StreamHandler.emit)
else:
    logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
'''



from django.db import connection
# middleware is supposed to be used within django application
# so we are not afraid to import django settings
from django.conf import settings as django_settings 
from djalog.conf import settings

import logging

class SQLLoggingMiddleware:
    def process_response(self, request, response):
        if not settings.LOG_SQL:
            # Middleware, even if is turned on at settings module, won't do
            # anything if LOG_SQL is set to False
            return response
        if not django_settings.DEBUG:
            return response
        if not connection.queries:
            return response
        total_time = sum([ float(q['time']) for q in connection.queries])
        
        for query in connection.queries:
            msg = "Query : %s\nTime elapsed: %s" % (query['sql'], query['time'])
            logging.sql(msg)
        logging.sql("Total SQL time for request: %s" % total_time)
        
        return response
        



'''
Concept and most code taken from one
of the snippets at djangosnippets.com
'''
import sys
import logging

DEFAULT_LEVEL = logging.INFO

FORMAT = "%(asctime)s [%(levelname)s] %(message)s" # (File: %(filename)s:%(lineno)d)"
DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

FILE_FORMAT = "%(asctime)s [%(levelname)s] %(message)s"
FILE_DATE_FORMAT = DATE_FORMAT

try:
    from django.conf import settings as django_settings
    if django_settings.configured is False:
        django_settings = object()
except ImportError:
    django_settings = object()

# Default settings or given with settings module
LOG_FORMAT = getattr(django_settings, 'DJALOG_FORMAT', FORMAT)
LOG_FORMAT_LINEO = getattr(django_settings, 'DJALOG_FORMAT_LINEO', False)
LOG_DATE_FORMAT = getattr(django_settings, 'DJALOG_DATE_FORMAT', DATE_FORMAT)
LOG_FILE = getattr(django_settings, 'DJALOG_FILE', None)
LOG_FILE_FORMAT = getattr(django_settings, 'DJALOG_FILE_FORMAT', FILE_FORMAT)
LOG_FILE_DATE_FORMAT = getattr(django_settings, 'DJALOG_FILE_DATE_FORMAT', FILE_DATE_FORMAT)
LOG_SQL = getattr(django_settings, 'DJALOG_SQL', False)
LOG_SQL_LEVEL = getattr(django_settings, 'DJALOG_SQL_LEVEL', 5)
LOG_LEVEL = getattr(django_settings, 'DJALOG_LEVEL', DEFAULT_LEVEL)
LOG_USE_COLORS = getattr(django_settings, 'DJALOG_USE_COLORS', False)
LOG_STREAM = getattr(django_settings, 'DJALOG_STREAM', sys.stdout)




import logging

from djalog.conf import configure
from djalog.loggers import DjalogLogger

VERSION = (0, 9, 2, 'stable')
__version__ = '.'.join(map(str, VERSION))

__all__ = ['configure', 'DjalogLogger', 'DjalogError']

configure()

def get_version():
    return __version__




import logging

from djalog.conf import settings
from djalog.loggers import DjalogLogger

def sql(msg):
    """
    Basic log sql function.
    """
    # Preformat message
    msg = msg.replace('SELECT', '\n\tSELECT')\
        .replace('FROM', '\n\tFROM')\
        .replace('ORDER BY', '\n\tORDER BY')\
        .replace('LIMIT', '\n\tLIMIT')\
        .replace('WHERE', '\n\tWHERE')\
        .replace('AND', '\n\tAND')\
        .replace('LEFT', '\n\tLEFT')\
        .replace('INNER', '\n\tINNER')\
        .replace('INSERT', '\n\tINNER')\
        .replace('DELETE', '\n\tINNER')
        
    DjalogLogger.log(logging.SQL, msg)

def init_sql_logging():
    """
    Initializes SQL logging settings or
    removes it from logging module.
    """
    # Clears SQL level / log_sql method
    def remattr(obj, attr):
        if hasattr(obj, attr):
            delattr(obj, attr)
    #DjalogLogger.debug("settings.LOG_SQL is set to %s" % settings.LOG_SQL)
    if settings.LOG_SQL:
        logging.addLevelName(settings.LOG_SQL_LEVEL, levelName='SQL')
        logging.SQL = settings.LOG_SQL_LEVEL
        logging.sql = sql
        DjalogLogger.sql = sql
    else:
        remattr(logging, 'SQL')
        remattr(logging, 'sql')
        remattr(DjalogLogger, 'sql')




import logging
import platform

from djalog import colored
from djalog.conf import settings
from djalog.formatters import ColoredFormatter

def ConsoleHandler():
    """
    Factory-method returns StreamHandler with
    dynamically created formatter according to
    djalog.conf.settings module.
    """
    ch = logging.StreamHandler()
    if settings.LOG_USE_COLORS:
        if platform.system().lower() not in colored.SUPPORTED_PLATFORMS:
            logging.warn("Cannot use colored output on this platform. Currently "
                "supported platforms are: %s." % ' '.join(colored.SUPPORTED_PLATFORMS))
        ch.setFormatter(ColoredFormatter(settings.LOG_FORMAT, settings.LOG_DATE_FORMAT))
    else:
        ch.setFormatter(logging.Formatter(settings.LOG_FORMAT, settings.LOG_DATE_FORMAT))
    return ch




import logging
import tempfile

from djalog.conf import settings
from djalog.handlers import ConsoleHandler

# Create loggers
DjalogLogger = logging.getLogger("DjalogLogger")
Logger = DjalogLogger # Temp

# Set level
DjalogLogger.setLevel(settings.LOG_LEVEL)

DjalogLogger.addHandler(ConsoleHandler())

# Set as default logger
if logging.root.name != 'DjalogLogger':
    logging.root = DjalogLogger

#logging.debug("Using logging level %s" % logging.root.level)




"""
    List of parameters with their default values:

    :param LOG_LEVEL=5 - sets level of logging for DjalogLogger

    :param LOG_FORMAT="%(asctime)s [%(levelname)s] %(message)s (File: %(filename)s:%(lineno)d)"
        this is format of the DjalogFileLogger

    :param LOG_DATE_FORMAT="%Y-%m-%d %H:%M:%S"
        this is date format used by %(asctime)s param inside normal format parameter

    :param LOG_FILE=None - a path to a file where
        logs should be stored if needed

    :param LOG_FILE_FORMAT="%(asctime)s [%(levelname)s] %(message)s"
        this if format for the DjalogFileLogger

    :param LOG_USE_COLORS=False - checks if logs (only main
        DjalogLogger) should use ANSI colors

    :param LOG_SQL=True - if True will use sql addtions from djalog.
        See djalog.init_sql_logging for more details

    :param LOG_SQL_LEVEL=5 - if using djalog sql logging additions
        it is the value of SQL logging level

    :param LOG_STREAM=sys.stdout - changes logger's stream

"""
import logging
import platform

from djalog import colored
from djalog.exc import DjalogError
from djalog.conf import settings
from djalog.loggers import DjalogLogger
from djalog.handlers import ConsoleHandler
from djalog.formatters import ColoredFormatter
from djalog.sqllogging import init_sql_logging

streamHandlerEmit = logging.StreamHandler.emit

def configure(**user_settings):
    """
    Configure or reconfigure djalog module.
    DjalogLogger is the name of main logger located
    at djalog.loggers. Additionally, logging.root becomes
    DjalogLogger by default..
    %s
    """ % __doc__
    safepoint = create_safepoint(settings)

    # First we update settings module
    for key, val in user_settings.items():
        if hasattr(settings, key):
            setattr(settings, key, val)
        else:
            raise DjalogError("Cannot set %s parameter! Available " \
                "parameters are described at djalog.configure.__doc__, "\
                "and here it is:\n%s" % (key, __doc__))

    DjalogLogger.setLevel(settings.LOG_LEVEL)
    # Clean up handlers
    for x in xrange(len(DjalogLogger.handlers)):
        DjalogLogger.handlers.pop(0)
    handler = ConsoleHandler()
    DjalogLogger.addHandler(handler)

    # Setup file handler
    if settings.LOG_FILE:
        # Try to regognize LOG_FILE if it is callable
        if callable(settings.LOG_FILE):
            # For instance, LOG_FILE may be set to tempfile.gettempdir
            settings.LOG_FILE = settings.LOG_FILE()
            DjalogLogger.debug("Setting log file to: %s" % settings.LOG_FILE)
        fileHandler = logging.FileHandler(settings.LOG_FILE)
        fileHandler.setFormatter(logging.Formatter(settings.LOG_FILE_FORMAT,
            settings.LOG_FILE_DATE_FORMAT))
        DjalogLogger.addHandler(fileHandler)

    # Initialize sql logging (or removes if LOG_SQL is False)
    #init_sql_logging(settings.LOG_SQL, level=settings.LOG_SQL_LEVEL)
    init_sql_logging()
    
def create_safepoint(settings_module):
    """
    Creates safepoint dict from Djalog settings module.
    """
    safepoint = {}
    for key in filter(str.isupper, dir(settings_module)):
        safepoint[key] = getattr(settings_module, key)
    return safepoint

def rollback_from_safepoint(safepoint):
    """
    Rolls changes back to ones from safepoint dict.
    """
    for key, val in safepoint.items():
        setattr(settings, key, val)

